<?php

namespace Sabre\VObject;

/**
 * This class contains the version number for the VObject package
 *
 * @copyright Copyright (C) 2011-2015 fruux GmbH (https://fruux.com/).
 * @author Evert Pot (http://evertpot.com/)
 * @license http://sabre.io/license/ Modified BSD License
 */
class Version {

    /**
     * Full version number
     */
    const VERSION = '2.1.7';

    /**
     * Stability : alpha, beta, stable
     */
    const STABILITY = 'stable';

}
